@@ -11,7 +11,6 @@
 #   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #   License for the specific language governing permissions and limitations
 #   under the License.
-#
 
 import copy
 import io
@@ -19,6 +18,7 @@ import os
 import tempfile
 from unittest import mock
 
+from cinderclient import api_versions
 from openstack import exceptions as sdk_exceptions
 from osc_lib.cli import format_columns
 from osc_lib import exceptions
@@ -26,33 +26,32 @@ from osc_lib import exceptions
 from openstackclient.image.v2 import image
 from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
 from openstackclient.tests.unit.image.v2 import fakes as image_fakes
+from openstackclient.tests.unit.volume.v3 import fakes as volume_fakes
 
 
-class TestImage(image_fakes.TestImagev2):
+class TestImage(image_fakes.TestImagev2, volume_fakes.TestVolume):
 
     def setUp(self):
         super(TestImage, self).setUp()
 
-        # Get shortcuts to the Mocks in image client
-        # SDK proxy mock
-        self.app.client_manager.image = mock.Mock()
+        # Get shortcuts to mocked image client
         self.client = self.app.client_manager.image
 
-        self.client.remove_member = mock.Mock()
-
-        self.client.create_image = mock.Mock()
-        self.client.update_image = mock.Mock()
-        self.image_members_mock = self.app.client_manager.image.image_members
-        self.image_tags_mock = self.app.client_manager.image.image_tags
-
         # Get shortcut to the Mocks in identity client
         self.project_mock = self.app.client_manager.identity.projects
         self.project_mock.reset_mock()
         self.domain_mock = self.app.client_manager.identity.domains
         self.domain_mock.reset_mock()
+        self.volumes_mock = self.app.client_manager.volume.volumes
+        fake_body = {
+            'os-volume_upload_image':
+                {'volume_type': {'name': 'fake_type'}}}
+        self.volumes_mock.upload_to_image.return_value = (
+            200, fake_body)
+        self.volumes_mock.reset_mock()
 
     def setup_images_mock(self, count):
-        images = image_fakes.FakeImage.create_images(count=count)
+        images = image_fakes.create_images(count=count)
 
         return images
 
@@ -65,7 +64,7 @@ class TestImageCreate(TestImage):
     def setUp(self):
         super(TestImageCreate, self).setUp()
 
-        self.new_image = image_fakes.FakeImage.create_one_image()
+        self.new_image = image_fakes.create_one_image()
         self.client.create_image.return_value = self.new_image
 
         self.project_mock.get.return_value = self.project
@@ -100,6 +99,7 @@ class TestImageCreate(TestImage):
         # ImageManager.create(name=, **)
         self.client.create_image.assert_called_with(
             name=self.new_image.name,
+            allow_duplicates=True,
             container_format=image.DEFAULT_CONTAINER_FORMAT,
             disk_format=image.DEFAULT_DISK_FORMAT,
         )
@@ -110,7 +110,7 @@ class TestImageCreate(TestImage):
         self.assertEqual(
             self.expected_columns,
             columns)
-        self.assertItemEqual(
+        self.assertCountEqual(
             self.expected_data,
             data)
 
@@ -152,6 +152,7 @@ class TestImageCreate(TestImage):
         # ImageManager.create(name=, **)
         self.client.create_image.assert_called_with(
             name=self.new_image.name,
+            allow_duplicates=True,
             container_format='ovf',
             disk_format='ami',
             min_disk=10,
@@ -164,7 +165,7 @@ class TestImageCreate(TestImage):
         self.assertEqual(
             self.expected_columns,
             columns)
-        self.assertItemEqual(
+        self.assertCountEqual(
             self.expected_data,
             data)
 
@@ -180,7 +181,7 @@ class TestImageCreate(TestImage):
             '--protected',
             '--private',
             '--project', 'unexist_owner',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('container_format', 'ovf'),
@@ -192,7 +193,7 @@ class TestImageCreate(TestImage):
             ('public', False),
             ('private', True),
             ('project', 'unexist_owner'),
-            ('name', image_fakes.image_name),
+            ('name', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -239,6 +240,7 @@ class TestImageCreate(TestImage):
         # ImageManager.create(name=, **)
         self.client.create_image.assert_called_with(
             name=self.new_image.name,
+            allow_duplicates=True,
             container_format=image.DEFAULT_CONTAINER_FORMAT,
             disk_format=image.DEFAULT_DISK_FORMAT,
             is_protected=self.new_image.is_protected,
@@ -246,13 +248,13 @@ class TestImageCreate(TestImage):
             Alpha='1',
             Beta='2',
             tags=self.new_image.tags,
-            filename=imagefile.name
+            filename=imagefile.name,
         )
 
         self.assertEqual(
             self.expected_columns,
             columns)
-        self.assertItemEqual(
+        self.assertCountEqual(
             self.expected_data,
             data)
 
@@ -288,18 +290,114 @@ class TestImageCreate(TestImage):
         # ImageManager.create(name=, **)
         self.client.create_image.assert_called_with(
             name=self.new_image.name,
+            allow_duplicates=True,
             container_format=image.DEFAULT_CONTAINER_FORMAT,
             disk_format=image.DEFAULT_DISK_FORMAT,
             use_import=True
         )
 
+    @mock.patch('osc_lib.utils.find_resource')
+    @mock.patch('openstackclient.image.v2.image.get_data_file')
+    def test_image_create_from_volume(self, mock_get_data_f, mock_get_vol):
+
+        fake_vol_id = 'fake-volume-id'
+        mock_get_data_f.return_value = (None, None)
+
+        class FakeVolume:
+            id = fake_vol_id
+
+        mock_get_vol.return_value = FakeVolume()
+
+        arglist = [
+            '--volume', fake_vol_id,
+            self.new_image.name,
+        ]
+        verifylist = [
+            ('name', self.new_image.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.upload_to_image.assert_called_with(
+            fake_vol_id,
+            False,
+            self.new_image.name,
+            'bare',
+            'raw'
+        )
+
+    @mock.patch('osc_lib.utils.find_resource')
+    @mock.patch('openstackclient.image.v2.image.get_data_file')
+    def test_image_create_from_volume_fail(self, mock_get_data_f,
+                                           mock_get_vol):
+
+        fake_vol_id = 'fake-volume-id'
+        mock_get_data_f.return_value = (None, None)
+
+        class FakeVolume:
+            id = fake_vol_id
+
+        mock_get_vol.return_value = FakeVolume()
+
+        arglist = [
+            '--volume', fake_vol_id,
+            self.new_image.name,
+            '--public'
+        ]
+        verifylist = [
+            ('name', self.new_image.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
+                          parsed_args)
+
+    @mock.patch('osc_lib.utils.find_resource')
+    @mock.patch('openstackclient.image.v2.image.get_data_file')
+    def test_image_create_from_volume_v31(self, mock_get_data_f,
+                                          mock_get_vol):
+
+        self.app.client_manager.volume.api_version = (
+            api_versions.APIVersion('3.1'))
+
+        fake_vol_id = 'fake-volume-id'
+        mock_get_data_f.return_value = (None, None)
+
+        class FakeVolume:
+            id = fake_vol_id
+
+        mock_get_vol.return_value = FakeVolume()
+
+        arglist = [
+            '--volume', fake_vol_id,
+            self.new_image.name,
+            '--public'
+        ]
+        verifylist = [
+            ('name', self.new_image.name),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.volumes_mock.upload_to_image.assert_called_with(
+            fake_vol_id,
+            False,
+            self.new_image.name,
+            'bare',
+            'raw',
+            visibility='public',
+            protected=False
+        )
+
 
 class TestAddProjectToImage(TestImage):
 
     project = identity_fakes.FakeProject.create_one_project()
     domain = identity_fakes.FakeDomain.create_one_domain()
-    _image = image_fakes.FakeImage.create_one_image()
-    new_member = image_fakes.FakeImage.create_one_image_member(
+    _image = image_fakes.create_one_image()
+    new_member = image_fakes.create_one_image_member(
         attrs={'image_id': _image.id,
                'member_id': project.id}
     )
@@ -431,7 +529,7 @@ class TestImageDelete(TestImage):
 
     def test_image_delete_multi_images_exception(self):
 
-        images = image_fakes.FakeImage.create_images(count=2)
+        images = image_fakes.create_images(count=2)
         arglist = [
             images[0].id,
             images[1].id,
@@ -463,7 +561,7 @@ class TestImageDelete(TestImage):
 
 class TestImageList(TestImage):
 
-    _image = image_fakes.FakeImage.create_one_image()
+    _image = image_fakes.create_one_image()
 
     columns = (
         'ID',
@@ -480,11 +578,7 @@ class TestImageList(TestImage):
     def setUp(self):
         super(TestImageList, self).setUp()
 
-        self.api_mock = mock.Mock()
-        self.api_mock.side_effect = [
-            [self._image], [],
-        ]
-        self.client.images = self.api_mock
+        self.client.images.side_effect = [[self._image], []]
 
         # Get the command object to test
         self.cmd = image.ListImage(self.app, None)
@@ -509,7 +603,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     def test_image_list_public_option(self):
         arglist = [
@@ -533,7 +627,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     def test_image_list_private_option(self):
         arglist = [
@@ -557,7 +651,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     def test_image_list_community_option(self):
         arglist = [
@@ -605,7 +699,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     def test_image_list_shared_member_status_option(self):
         arglist = [
@@ -693,7 +787,7 @@ class TestImageList(TestImage):
             self._image.owner_id,
             format_columns.ListColumn(self._image.tags),
         ), )
-        self.assertListItemEqual(datalist, tuple(data))
+        self.assertCountEqual(datalist, tuple(data))
 
     @mock.patch('osc_lib.api.utils.simple_filter')
     def test_image_list_property_option(self, sf_mock):
@@ -721,7 +815,7 @@ class TestImageList(TestImage):
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     @mock.patch('osc_lib.utils.sort_items')
     def test_image_list_sort_option(self, si_mock):
@@ -743,7 +837,7 @@ class TestImageList(TestImage):
             str,
         )
         self.assertEqual(self.columns, columns)
-        self.assertListItemEqual(self.datalist, tuple(data))
+        self.assertCountEqual(self.datalist, tuple(data))
 
     def test_image_list_limit_option(self):
         ret_limit = 1
@@ -765,20 +859,30 @@ class TestImageList(TestImage):
         self.assertEqual(self.columns, columns)
         self.assertEqual(ret_limit, len(tuple(data)))
 
-    @mock.patch('osc_lib.utils.find_resource')
-    def test_image_list_marker_option(self, fr_mock):
-        # tangchen: Since image_fakes.IMAGE is a dict, it cannot offer a .id
-        #           operation. Will fix this by using FakeImage class instead
-        #           of IMAGE dict.
+    def test_image_list_project_option(self):
         self.client.find_image = mock.Mock(return_value=self._image)
-#         fr_mock.return_value = mock.Mock()
-#         fr_mock.return_value.id = image_fakes.image_id
-
         arglist = [
-            '--marker', image_fakes.image_name,
+            '--project', 'nova',
         ]
         verifylist = [
-            ('marker', image_fakes.image_name),
+            ('project', 'nova'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+
+        self.assertEqual(self.columns, columns)
+        self.assertCountEqual(self.datalist, tuple(data))
+
+    @mock.patch('osc_lib.utils.find_resource')
+    def test_image_list_marker_option(self, fr_mock):
+        self.client.find_image = mock.Mock(return_value=self._image)
+
+        arglist = [
+            '--marker', 'graven',
+        ]
+        verifylist = [
+            ('marker', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -787,7 +891,7 @@ class TestImageList(TestImage):
             marker=self._image.id,
         )
 
-        self.client.find_image.assert_called_with(image_fakes.image_name)
+        self.client.find_image.assert_called_with('graven')
 
     def test_image_list_name_option(self):
         arglist = [
@@ -818,6 +922,20 @@ class TestImageList(TestImage):
             status='active'
         )
 
+    def test_image_list_hidden_option(self):
+        arglist = [
+            '--hidden',
+        ]
+        verifylist = [
+            ('hidden', True),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        columns, data = self.cmd.take_action(parsed_args)
+        self.client.images.assert_called_with(
+            is_hidden=True
+        )
+
     def test_image_list_tag_option(self):
         arglist = [
             '--tag', 'abc',
@@ -836,8 +954,8 @@ class TestImageList(TestImage):
 class TestListImageProjects(TestImage):
 
     project = identity_fakes.FakeProject.create_one_project()
-    _image = image_fakes.FakeImage.create_one_image()
-    member = image_fakes.FakeImage.create_one_image_member(
+    _image = image_fakes.create_one_image()
+    member = image_fakes.create_one_image_member(
         attrs={'image_id': _image.id,
                'member_id': project.id}
     )
@@ -887,7 +1005,7 @@ class TestRemoveProjectImage(TestImage):
     def setUp(self):
         super(TestRemoveProjectImage, self).setUp()
 
-        self._image = image_fakes.FakeImage.create_one_image()
+        self._image = image_fakes.create_one_image()
         # This is the return value for utils.find_resource()
         self.client.find_image.return_value = self._image
 
@@ -946,7 +1064,7 @@ class TestImageSet(TestImage):
 
     project = identity_fakes.FakeProject.create_one_project()
     domain = identity_fakes.FakeDomain.create_one_domain()
-    _image = image_fakes.FakeImage.create_one_image({'tags': []})
+    _image = image_fakes.create_one_image({'tags': []})
 
     def setUp(self):
         super(TestImageSet, self).setUp()
@@ -966,22 +1084,24 @@ class TestImageSet(TestImage):
 
     def test_image_set_no_options(self):
         arglist = [
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
-            ('image', image_fakes.image_id)
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c')
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
         result = self.cmd.take_action(parsed_args)
 
         self.assertIsNone(result)
-
-        self.image_members_mock.update.assert_not_called()
+        # we'll have called this but not set anything
+        self.app.client_manager.image.update_image.called_once_with(
+            self._image.id,
+        )
 
     def test_image_set_membership_option_accept(self):
-        membership = image_fakes.FakeImage.create_one_image_member(
-            attrs={'image_id': image_fakes.image_id,
+        membership = image_fakes.create_one_image_member(
+            attrs={'image_id': '0f41529e-7c12-4de8-be2d-181abb825b3c',
                    'member_id': self.project.id}
         )
         self.client.update_member.return_value = membership
@@ -991,9 +1111,7 @@ class TestImageSet(TestImage):
             self._image.id,
         ]
         verifylist = [
-            ('accept', True),
-            ('reject', False),
-            ('pending', False),
+            ('membership', 'accepted'),
             ('image', self._image.id)
         ]
 
@@ -1011,21 +1129,19 @@ class TestImageSet(TestImage):
         self.client.update_image.assert_called_with(self._image.id)
 
     def test_image_set_membership_option_reject(self):
-        membership = image_fakes.FakeImage.create_one_image_member(
-            attrs={'image_id': image_fakes.image_id,
+        membership = image_fakes.create_one_image_member(
+            attrs={'image_id': '0f41529e-7c12-4de8-be2d-181abb825b3c',
                    'member_id': self.project.id}
         )
         self.client.update_member.return_value = membership
 
         arglist = [
             '--reject',
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
-            ('accept', False),
-            ('reject', True),
-            ('pending', False),
-            ('image', image_fakes.image_id)
+            ('membership', 'rejected'),
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c')
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -1042,21 +1158,19 @@ class TestImageSet(TestImage):
         self.client.update_image.assert_called_with(self._image.id)
 
     def test_image_set_membership_option_pending(self):
-        membership = image_fakes.FakeImage.create_one_image_member(
-            attrs={'image_id': image_fakes.image_id,
+        membership = image_fakes.create_one_image_member(
+            attrs={'image_id': '0f41529e-7c12-4de8-be2d-181abb825b3c',
                    'member_id': self.project.id}
         )
         self.client.update_member.return_value = membership
 
         arglist = [
             '--pending',
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
-            ('accept', False),
-            ('reject', False),
-            ('pending', True),
-            ('image', image_fakes.image_id)
+            ('membership', 'pending'),
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c')
         ]
 
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
@@ -1116,11 +1230,11 @@ class TestImageSet(TestImage):
 
         arglist = [
             '--project', 'unexist_owner',
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
             ('project', 'unexist_owner'),
-            ('image', image_fakes.image_id),
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1132,14 +1246,14 @@ class TestImageSet(TestImage):
         arglist = [
             '--protected',
             '--private',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('protected', True),
             ('unprotected', False),
             ('public', False),
             ('private', True),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1160,14 +1274,14 @@ class TestImageSet(TestImage):
         arglist = [
             '--unprotected',
             '--public',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('protected', False),
             ('unprotected', True),
             ('public', True),
             ('private', False),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1188,11 +1302,11 @@ class TestImageSet(TestImage):
         arglist = [
             '--property', 'Alpha=1',
             '--property', 'Beta=2',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('properties', {'Alpha': '1', 'Beta': '2'}),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1217,7 +1331,7 @@ class TestImageSet(TestImage):
             '--os-distro', 'cpm',
             '--os-version', '2.2H',
             '--ramdisk-id', 'xyzpdq',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('architecture', 'z80'),
@@ -1226,7 +1340,7 @@ class TestImageSet(TestImage):
             ('os_distro', 'cpm'),
             ('os_version', '2.2H'),
             ('ramdisk_id', 'xyzpdq'),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1250,11 +1364,11 @@ class TestImageSet(TestImage):
     def test_image_set_tag(self):
         arglist = [
             '--tag', 'test-tag',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('tags', ['test-tag']),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1274,11 +1388,11 @@ class TestImageSet(TestImage):
         arglist = [
             '--tag', 'test-tag',
             '--activate',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('tags', ['test-tag']),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1302,11 +1416,11 @@ class TestImageSet(TestImage):
         arglist = [
             '--tag', 'test-tag',
             '--deactivate',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('tags', ['test-tag']),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1332,11 +1446,11 @@ class TestImageSet(TestImage):
         self.client.find_image.return_value = old_image
         arglist = [
             '--tag', 'test-tag',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('tags', ['test-tag']),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1358,11 +1472,11 @@ class TestImageSet(TestImage):
         self.client.find_image.return_value = old_image
         arglist = [
             '--tag', 'old1',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('tags', ['old1']),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1382,11 +1496,11 @@ class TestImageSet(TestImage):
 
         arglist = [
             '--visibility', '1-mile',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('visibility', '1-mile'),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1398,12 +1512,12 @@ class TestImageSet(TestImage):
         arglist = [
             '--min-disk', '0',
             '--min-ram', '0',
-            image_fakes.image_name,
+            'graven',
         ]
         verifylist = [
             ('min_disk', 0),
             ('min_ram', 0),
-            ('image', image_fakes.image_name),
+            ('image', 'graven'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1420,13 +1534,67 @@ class TestImageSet(TestImage):
         )
         self.assertIsNone(result)
 
+    def test_image_set_hidden(self):
+        arglist = [
+            '--hidden',
+            '--public',
+            'graven',
+        ]
+        verifylist = [
+            ('hidden', True),
+            ('public', True),
+            ('private', False),
+            ('image', 'graven'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'is_hidden': True,
+            'visibility': 'public',
+        }
+        # ImageManager.update(image, **kwargs)
+        self.client.update_image.assert_called_with(
+            self._image.id,
+            **kwargs
+        )
+        self.assertIsNone(result)
+
+    def test_image_set_unhidden(self):
+        arglist = [
+            '--unhidden',
+            '--public',
+            'graven',
+        ]
+        verifylist = [
+            ('hidden', False),
+            ('public', True),
+            ('private', False),
+            ('image', 'graven'),
+        ]
+        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
+
+        result = self.cmd.take_action(parsed_args)
+
+        kwargs = {
+            'is_hidden': False,
+            'visibility': 'public',
+        }
+        # ImageManager.update(image, **kwargs)
+        self.client.update_image.assert_called_with(
+            self._image.id,
+            **kwargs
+        )
+        self.assertIsNone(result)
+
 
 class TestImageShow(TestImage):
 
-    new_image = image_fakes.FakeImage.create_one_image(
+    new_image = image_fakes.create_one_image(
         attrs={'size': 1000})
 
-    _data = image_fakes.FakeImage.create_one_image()
+    _data = image_fakes.create_one_image()
 
     columns = (
         'id', 'name', 'owner', 'protected', 'tags', 'visibility'
@@ -1451,10 +1619,10 @@ class TestImageShow(TestImage):
 
     def test_image_show(self):
         arglist = [
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
-            ('image', image_fakes.image_id),
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c'),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1463,12 +1631,12 @@ class TestImageShow(TestImage):
         # data to be shown.
         columns, data = self.cmd.take_action(parsed_args)
         self.client.find_image.assert_called_with(
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
             ignore_missing=False
         )
 
         self.assertEqual(self.columns, columns)
-        self.assertItemEqual(self.data, data)
+        self.assertCountEqual(self.data, data)
 
     def test_image_show_human_readable(self):
         self.client.find_image.return_value = self.new_image
@@ -1497,15 +1665,16 @@ class TestImageShow(TestImage):
 
 class TestImageUnset(TestImage):
 
-    attrs = {}
-    attrs['tags'] = ['test']
-    attrs['prop'] = 'test'
-    attrs['prop2'] = 'fake'
-    image = image_fakes.FakeImage.create_one_image(attrs)
-
     def setUp(self):
         super(TestImageUnset, self).setUp()
 
+        attrs = {}
+        attrs['tags'] = ['test']
+        attrs['hw_rng_model'] = 'virtio'
+        attrs['prop'] = 'test'
+        attrs['prop2'] = 'fake'
+        self.image = image_fakes.create_one_image(attrs)
+
         self.client.find_image.return_value = self.image
         self.client.remove_tag.return_value = self.image
         self.client.update_image.return_value = self.image
@@ -1515,10 +1684,10 @@ class TestImageUnset(TestImage):
 
     def test_image_unset_no_options(self):
         arglist = [
-            image_fakes.image_id,
+            '0f41529e-7c12-4de8-be2d-181abb825b3c',
         ]
         verifylist = [
-            ('image', image_fakes.image_id)
+            ('image', '0f41529e-7c12-4de8-be2d-181abb825b3c')
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1548,22 +1717,20 @@ class TestImageUnset(TestImage):
     def test_image_unset_property_option(self):
 
         arglist = [
+            '--property', 'hw_rng_model',
             '--property', 'prop',
             self.image.id,
         ]
 
         verifylist = [
-            ('properties', ['prop']),
+            ('properties', ['hw_rng_model', 'prop']),
             ('image', self.image.id)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        kwargs = {}
         self.client.update_image.assert_called_with(
-            self.image,
-            properties={'prop2': 'fake'},
-            **kwargs)
+            self.image, properties={'prop2': 'fake'})
 
         self.assertIsNone(result)
 
@@ -1571,23 +1738,21 @@ class TestImageUnset(TestImage):
 
         arglist = [
             '--tag', 'test',
+            '--property', 'hw_rng_model',
             '--property', 'prop',
             self.image.id,
         ]
 
         verifylist = [
             ('tags', ['test']),
-            ('properties', ['prop']),
+            ('properties', ['hw_rng_model', 'prop']),
             ('image', self.image.id)
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
         result = self.cmd.take_action(parsed_args)
 
-        kwargs = {}
         self.client.update_image.assert_called_with(
-            self.image,
-            properties={'prop2': 'fake'},
-            **kwargs)
+            self.image, properties={'prop2': 'fake'})
 
         self.client.remove_tag.assert_called_with(
             self.image.id, 'test'
@@ -1597,7 +1762,7 @@ class TestImageUnset(TestImage):
 
 class TestImageSave(TestImage):
 
-    image = image_fakes.FakeImage.create_one_image({})
+    image = image_fakes.create_one_image({})
 
     def setUp(self):
         super(TestImageSave, self).setUp()
@@ -1614,7 +1779,7 @@ class TestImageSave(TestImage):
 
         verifylist = [
             ('file', '/path/to/file'),
-            ('image', self.image.id)
+            ('image', self.image.id),
         ]
         parsed_args = self.check_parser(self.cmd, arglist, verifylist)
 
@@ -1622,6 +1787,7 @@ class TestImageSave(TestImage):
 
         self.client.download_image.assert_called_once_with(
             self.image.id,
+            stream=True,
             output='/path/to/file')
 
 
